@@ -30,6 +30,7 @@
 #include "gl-eventviewrow.h"
 #include "gl-journal-model.h"
 #include "gl-util.h"
+#include "gl-searchpopover.h"
 
 struct _GlEventViewList
 {
@@ -50,6 +51,11 @@ typedef struct
     GtkWidget *event_search;
     GtkWidget *event_scrolled;
     GtkWidget *search_entry;
+    GtkWidget *search_dropdown_button;
+    GtkWidget *search_popover;
+    GlSearchPopoverJournalFieldFilter journal_search_field;
+    GlQuerySearchType search_type;
+    GlSearchPopoverJournalTimestampRange journal_timestamp_range;
     gchar *search_text;
     const gchar *boot_match;
 } GlEventViewListPrivate;
@@ -373,18 +379,10 @@ get_current_boot_id (const gchar *boot_match)
 
 static void
 query_add_category_matches (GlQuery *query,
-                            GlCategoryList *list,
-                            const gchar *boot_match)
+                            GlCategoryList *list)
 {
-    gchar *boot_id;
     GlCategoryListFilter filter;
 
-    /* Get current boot id */
-    boot_id = get_current_boot_id (boot_match);
-
-    /* Add boot match for all the categories */
-    gl_query_add_match (query, "_BOOT_ID", boot_id, SEARCH_TYPE_EXACT);
-
     /* Add exact matches according to selected category */
     filter = gl_category_list_get_category (list);
 
@@ -393,10 +391,10 @@ query_add_category_matches (GlQuery *query,
         case GL_CATEGORY_LIST_FILTER_IMPORTANT:
             {
               /* Alert or emergency priority. */
-              gl_query_add_match (query, "PRIORITY", "0", SEARCH_TYPE_EXACT);
-              gl_query_add_match (query, "PRIORITY", "1", SEARCH_TYPE_EXACT);
-              gl_query_add_match (query, "PRIORITY", "2", SEARCH_TYPE_EXACT);
-              gl_query_add_match (query, "PRIORITY", "3", SEARCH_TYPE_EXACT);
+              gl_query_add_match (query, "PRIORITY", "0", GL_QUERY_SEARCH_TYPE_EXACT);
+              gl_query_add_match (query, "PRIORITY", "1", GL_QUERY_SEARCH_TYPE_EXACT);
+              gl_query_add_match (query, "PRIORITY", "2", GL_QUERY_SEARCH_TYPE_EXACT);
+              gl_query_add_match (query, "PRIORITY", "3", GL_QUERY_SEARCH_TYPE_EXACT);
             }
             break;
 
@@ -414,10 +412,10 @@ query_add_category_matches (GlQuery *query,
 
                 uid_str = get_uid_match_field_value ();
 
-                gl_query_add_match (query, "_TRANSPORT", "journal", SEARCH_TYPE_EXACT);
-                gl_query_add_match (query, "_TRANSPORT", "stdout", SEARCH_TYPE_EXACT);
-                gl_query_add_match (query, "_TRANSPORT", "syslog", SEARCH_TYPE_EXACT);
-                gl_query_add_match (query, "_UID", uid_str, SEARCH_TYPE_EXACT);
+                gl_query_add_match (query, "_TRANSPORT", "journal", GL_QUERY_SEARCH_TYPE_EXACT);
+                gl_query_add_match (query, "_TRANSPORT", "stdout", GL_QUERY_SEARCH_TYPE_EXACT);
+                gl_query_add_match (query, "_TRANSPORT", "syslog", GL_QUERY_SEARCH_TYPE_EXACT);
+                gl_query_add_match (query, "_UID", uid_str, GL_QUERY_SEARCH_TYPE_EXACT);
 
                 g_free (uid_str);
             }
@@ -425,44 +423,182 @@ query_add_category_matches (GlQuery *query,
 
         case GL_CATEGORY_LIST_FILTER_SYSTEM:
             {
-                gl_query_add_match (query, "_TRANSPORT", "kernel", SEARCH_TYPE_EXACT);
+                gl_query_add_match (query, "_TRANSPORT", "kernel", GL_QUERY_SEARCH_TYPE_EXACT);
             }
             break;
 
         case GL_CATEGORY_LIST_FILTER_HARDWARE:
             {
-                gl_query_add_match (query, "_TRANSPORT", "kernel", SEARCH_TYPE_EXACT);
-                gl_query_add_match ( query, "_KERNEL_DEVICE", NULL, SEARCH_TYPE_EXACT);
+                gl_query_add_match (query, "_TRANSPORT", "kernel", GL_QUERY_SEARCH_TYPE_EXACT);
+                gl_query_add_match ( query, "_KERNEL_DEVICE", NULL, GL_QUERY_SEARCH_TYPE_EXACT);
             }
             break;
 
         case GL_CATEGORY_LIST_FILTER_SECURITY:
             {
-                gl_query_add_match (query, "_AUDIT_SESSION", NULL, SEARCH_TYPE_EXACT);
+                gl_query_add_match (query, "_AUDIT_SESSION", NULL, GL_QUERY_SEARCH_TYPE_EXACT);
             }
             break;
 
         default:
             g_assert_not_reached ();
     }
-
-    g_free (boot_id);
 }
 
 static void
 query_add_search_matches (GlQuery *query,
-                          const gchar *search_text)
+                          const gchar *search_text,
+                          GlSearchPopoverJournalFieldFilter journal_search_field,
+                          GlQuerySearchType search_type)
 {
-    /* Add substring matches */
-    gl_query_add_match (query, "_PID", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "_UID", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "_GID", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "MESSAGE", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "_COMM", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "_SYSTEMD_UNIT", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "_KERNEL_DEVICE", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "_AUDIT_SESSION", search_text, SEARCH_TYPE_SUBSTRING);
-    gl_query_add_match (query, "_EXE", search_text, SEARCH_TYPE_SUBSTRING);
+    switch (journal_search_field)
+    {
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_ALL_AVAILABLE_FIELDS:
+            gl_query_add_match (query, "_PID", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "_UID", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "_GID", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "MESSAGE", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "_COMM", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "_SYSTEMD_UNIT", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "_KERNEL_DEVICE", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "_AUDIT_SESSION", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            gl_query_add_match (query, "_EXE", search_text, GL_QUERY_SEARCH_TYPE_SUBSTRING);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_PID:
+            gl_query_add_match (query, "_PID", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_UID:
+            gl_query_add_match (query, "_UID", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_GID:
+            gl_query_add_match (query, "_GID", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_MESSAGE:
+            gl_query_add_match (query, "MESSAGE", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_PROCESS_NAME:
+            gl_query_add_match (query, "_COMM", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_SYSTEMD_UNIT:
+            gl_query_add_match (query, "_SYSTEMD_UNIT", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_KERNEL_DEVICE:
+            gl_query_add_match (query, "_KERNEL_DEVICE", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_AUDIT_SESSION:
+            gl_query_add_match (query, "_AUDIT_SESSION", search_text, search_type);
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_FIELD_FILTER_EXECUTABLE_PATH:
+            gl_query_add_match (query, "_EXE", search_text, search_type);
+            break;
+    }
+}
+
+static void
+query_set_day_timestamps (GlQuery *query,
+                          gint start_day_offset,
+                          gint end_day_offset)
+{
+    GDateTime *now;
+    GDateTime *today_start;
+    GDateTime *today_end;
+    guint64 start_timestamp;
+    guint64 end_timestamp;
+
+    now = g_date_time_new_now_local();
+
+    today_start = g_date_time_new_local (g_date_time_get_year (now),
+                                         g_date_time_get_month (now),
+                                         g_date_time_get_day_of_month (now) - start_day_offset,
+                                         23,
+                                         59,
+                                         59.0);
+
+    start_timestamp = g_date_time_to_unix (today_start) * G_USEC_PER_SEC;
+
+    today_end = g_date_time_new_local (g_date_time_get_year (now),
+                                       g_date_time_get_month (now),
+                                       g_date_time_get_day_of_month (now) - end_day_offset,
+                                       0,
+                                       0,
+                                       0.0);
+
+    end_timestamp = g_date_time_to_unix (today_end) * G_USEC_PER_SEC;
+
+    gl_query_set_journal_timestamp_range (query, start_timestamp, end_timestamp);
+
+    g_date_time_unref (now);
+    g_date_time_unref (today_start);
+    g_date_time_unref (today_end);
+}
+
+static void
+query_add_journal_range_filter (GlQuery *query,
+                                GlEventViewList *view)
+{
+    GlEventViewListPrivate *priv;
+
+    priv = gl_event_view_list_get_instance_private (view);
+    /* Add range filters */
+    switch (priv->journal_timestamp_range)
+    {
+        case GL_SEARCH_POPOVER_JOURNAL_TIMESTAMP_RANGE_CURRENT_BOOT:
+        {
+            /* Get current boot id */
+            gchar *boot_match;
+
+            boot_match = get_current_boot_id (priv->boot_match);
+            gl_query_add_match (query, "_BOOT_ID", boot_match, GL_QUERY_SEARCH_TYPE_EXACT);
+
+            g_free (boot_match);
+        }
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_TIMESTAMP_RANGE_PREVIOUS_BOOT:
+        {
+            GArray *boot_ids;
+            GlJournalBootID *boot_id;
+
+            boot_ids = gl_event_view_list_get_boot_ids (view);
+
+            boot_id = &g_array_index (boot_ids, GlJournalBootID, boot_ids->len - 2);
+
+            gl_query_set_journal_timestamp_range (query, boot_id->realtime_last, boot_id->realtime_first);
+        }
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_TIMESTAMP_RANGE_TODAY:
+
+            query_set_day_timestamps (query, 0, 0);
+
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_TIMESTAMP_RANGE_YESTERDAY:
+
+            query_set_day_timestamps (query, 1, 1);
+
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_TIMESTAMP_RANGE_LAST_3_DAYS:
+
+            query_set_day_timestamps (query, 0, 2);
+
+            break;
+        case GL_SEARCH_POPOVER_JOURNAL_TIMESTAMP_RANGE_CUSTOM:
+        {
+            GlSearchPopover *popover;
+            guint64 custom_start_timestamp;
+            guint64 custom_end_timestamp;
+
+            popover = GL_SEARCH_POPOVER (priv->search_popover);
+
+            custom_start_timestamp = gl_search_popover_get_custom_start_timestamp (popover);
+            custom_end_timestamp = gl_search_popover_get_custom_end_timestamp (popover);
+
+            gl_query_set_journal_timestamp_range (query, custom_start_timestamp, custom_end_timestamp);
+            break;
+        }
+        default:
+
+            /* By default, search the entire journal */
+            break;
+    }
 }
 
 /* Create query object according to selected category */
@@ -479,9 +615,14 @@ create_query_object (GlEventViewList *view)
     /* Create new query object */
     query = gl_query_new ();
 
-    query_add_category_matches (query, list, priv->boot_match);
+    /* Set journal timestamp range */
+    query_add_journal_range_filter (query, view);
 
-    query_add_search_matches (query, priv->search_text);
+    query_add_category_matches (query, list);
+
+    query_add_search_matches (query, priv->search_text, priv->journal_search_field, priv->search_type);
+
+    gl_query_set_search_type (query, priv->search_type);
 
     return query;
 }
@@ -516,15 +657,17 @@ void
 gl_event_view_list_view_boot (GlEventViewList *view, const gchar *match)
 {
     GlEventViewListPrivate *priv;
-    GlCategoryList *categories;
+    GlSearchPopover *popover;
 
     g_return_if_fail (GL_EVENT_VIEW_LIST (view));
 
     priv = gl_event_view_list_get_instance_private (view);
-    categories = GL_CATEGORY_LIST (priv->categories);
+    popover = GL_SEARCH_POPOVER (priv->search_popover);
     priv->boot_match = match;
 
-    on_notify_category (categories, NULL, view);
+    /* Make search popover journal timestamp range label consistent with
+       event-toolbar boot selection menu */
+    gl_search_popover_set_journal_timestamp_range_current_boot (popover);
 }
 
 static gint
@@ -675,6 +818,78 @@ gl_event_list_view_edge_reached (GtkScrolledWindow *scrolled,
         gl_journal_model_fetch_more_entries (priv->journal_model, FALSE);
 }
 
+static void
+search_popover_journal_search_field_changed (GlSearchPopover *popover,
+                                             GParamSpec *psec,
+                                             GlEventViewList *view)
+{
+    GlEventViewListPrivate *priv = gl_event_view_list_get_instance_private (view);
+    GlQuery *query;
+
+    priv->journal_search_field = gl_search_popover_get_journal_search_field (popover);
+
+    query = create_query_object (view);
+
+    gl_journal_model_take_query (priv->journal_model, query);
+}
+
+static void
+search_popover_search_type_changed (GlSearchPopover *popover,
+                                    GParamSpec *psec,
+                                    GlEventViewList *view)
+{
+    GlEventViewListPrivate *priv = gl_event_view_list_get_instance_private (view);
+    GlQuery *query;
+
+    priv->search_type = gl_search_popover_get_query_search_type (popover);
+
+    query = create_query_object (view);
+
+    gl_journal_model_take_query (priv->journal_model, query);
+}
+
+static void
+search_popover_journal_timestamp_range_changed (GlSearchPopover *popover,
+                                                GParamSpec *psec,
+                                                GlEventViewList *view)
+{
+    GlEventViewListPrivate *priv = gl_event_view_list_get_instance_private (view);
+    GlQuery *query;
+
+    priv->journal_timestamp_range = gl_search_popover_get_journal_timestamp_range (popover);
+
+    query = create_query_object (view);
+
+    gl_journal_model_take_query (priv->journal_model, query);
+}
+
+/* Get the view elements from ui file and link it with the drop down button */
+static void
+set_up_search_popover (GlEventViewList *view)
+{
+
+    GlEventViewListPrivate *priv;
+
+    priv = gl_event_view_list_get_instance_private (view);
+
+    priv->search_popover = gl_search_popover_new ();
+
+    /* Grab/Remove keyboard focus from popover menu when it is opened or closed */
+    g_signal_connect (priv->search_popover, "show", (GCallback) gtk_widget_grab_focus, NULL);
+    g_signal_connect_swapped (priv->search_popover, "closed", (GCallback) gtk_widget_grab_focus, view);
+
+    g_signal_connect (priv->search_popover, "notify::journal-search-field",
+                      G_CALLBACK (search_popover_journal_search_field_changed), view);
+    g_signal_connect (priv->search_popover, "notify::search-type",
+                      G_CALLBACK (search_popover_search_type_changed), view);
+    g_signal_connect (priv->search_popover, "notify::journal-timestamp-range",
+                      G_CALLBACK (search_popover_journal_timestamp_range_changed), view);
+
+    /* Link the drop down button with search popover */
+    gtk_menu_button_set_popover (GTK_MENU_BUTTON (priv->search_dropdown_button),
+                                 priv->search_popover);
+}
+
 static void
 gl_event_view_list_finalize (GObject *object)
 {
@@ -708,6 +923,8 @@ gl_event_view_list_class_init (GlEventViewListClass *klass)
                                                   event_scrolled);
     gtk_widget_class_bind_template_child_private (widget_class, GlEventViewList,
                                                   search_entry);
+    gtk_widget_class_bind_template_child_private (widget_class, GlEventViewList,
+                                                  search_dropdown_button);
 
     gtk_widget_class_bind_template_callback (widget_class,
                                              on_search_entry_changed);
@@ -758,29 +975,12 @@ gl_event_view_list_init (GlEventViewList *view)
     priv->clock_format = g_settings_get_enum (settings, CLOCK_FORMAT);
     g_object_unref (settings);
 
+    set_up_search_popover (view);
+
     g_signal_connect (categories, "notify::category", G_CALLBACK (on_notify_category),
                       view);
 }
 
-void
-gl_event_view_list_search (GlEventViewList *view,
-                           const gchar *needle)
-{
-    GlEventViewListPrivate *priv;
-
-    g_return_if_fail (GL_EVENT_VIEW_LIST (view));
-
-    priv = gl_event_view_list_get_instance_private (view);
-
-    g_free (priv->search_text);
-    priv->search_text = g_strdup (needle);
-
-    /* for search, we need all entries - tell the model to fetch them */
-    gl_journal_model_fetch_more_entries (priv->journal_model, TRUE);
-
-    gtk_list_box_invalidate_filter (priv->entries_box);
-}
-
 GtkWidget *
 gl_event_view_list_new (void)
 {
