@@ -45,6 +45,7 @@ struct _GlJournalModel
 
 static void gl_journal_model_interface_init (GListModelInterface *iface);
 static gboolean search_in_entry (GlJournalEntry *entry, GlQuery *query);
+static gboolean gl_query_check_journal_end (GlQuery *query, GlJournalEntry *entry);
 
 G_DEFINE_TYPE_WITH_CODE (GlJournalModel, gl_journal_model, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, gl_journal_model_interface_init))
@@ -76,7 +77,7 @@ gl_journal_model_fetch_idle (gpointer user_data)
     g_assert (model->n_entries_to_fetch > 0);
 
     last = model->entries->len;
-    if ((entry = gl_journal_previous (model->journal)))
+    if ((entry = gl_journal_previous (model->journal)) && gl_query_check_journal_end (model->query, entry))
     {
         if (search_in_entry (entry, model->query))
         {
@@ -245,6 +246,9 @@ gl_query_new (void)
     query = g_slice_new (GlQuery);
 
     query->queryitems = g_ptr_array_new_with_free_func ((GDestroyNotify) gl_query_item_free);
+    query->search_type = GL_QUERY_SEARCH_TYPE_SUBSTRING;
+    query->start_timestamp = 0;
+    query->end_timestamp = 0;
 
     return query;
 }
@@ -265,6 +269,12 @@ gl_query_item_new (const gchar *field_name,
     return queryitem;
 }
 
+void
+gl_query_set_search_type (GlQuery *query, GlQuerySearchType search_type)
+{
+    query->search_type = search_type;
+}
+
 static gchar *
 gl_query_item_create_match_string (GlQueryItem *queryitem)
 {
@@ -287,7 +297,7 @@ populate_exact_matches (GlQueryItem *queryitem, GPtrArray *matches)
 {
     gchar *match;
 
-    if (queryitem->search_type == SEARCH_TYPE_EXACT)
+    if (queryitem->search_type == GL_QUERY_SEARCH_TYPE_EXACT)
     {
         match = gl_query_item_create_match_string (queryitem);
 
@@ -311,7 +321,7 @@ gl_query_get_exact_matches (GlQuery *query)
 static void
 populate_substring_matches (GlQueryItem *queryitem, GPtrArray *matches)
 {
-    if (queryitem->search_type == SEARCH_TYPE_SUBSTRING)
+    if (queryitem->search_type == GL_QUERY_SEARCH_TYPE_SUBSTRING)
     {
         g_ptr_array_add (matches, queryitem);
     }
@@ -339,8 +349,26 @@ gl_journal_model_process_query (GlJournalModel *model)
     /* Set the exact matches first */
     category_matches = gl_query_get_exact_matches (model->query);
 
+    /* Get the search string of the exact match field */
+    if (model->query->search_type == GL_QUERY_SEARCH_TYPE_EXACT)
+    {
+        gchar *search_match;
+        gchar *field_value_pos;
+
+        /* Get the search match string */
+        search_match = g_ptr_array_index (category_matches, category_matches->len - 1);
+
+        field_value_pos = strchr (search_match, '=');
+
+        /* If it has invalid string value remove it from the matches */
+        if (!field_value_pos || !*(field_value_pos + 1))
+            g_ptr_array_remove (category_matches, search_match);
+    }
+
     gl_journal_set_matches (model->journal, category_matches);
 
+    gl_journal_set_start_position (model->journal, model->query->start_timestamp);
+
     /* Start re-population of the journal */
     gl_journal_model_fetch_more_entries (model, FALSE);
 
@@ -402,6 +430,38 @@ gl_query_add_match (GlQuery *query,
     g_ptr_array_add (query->queryitems, queryitem);
 }
 
+/* Check if current entry timestamp is less than the end timestamp */
+static gboolean
+gl_journal_entry_check_journal_end (GlJournalEntry *entry,
+                                    guint64 end_timestamp)
+{
+    guint64 entry_timestamp = gl_journal_entry_get_timestamp (entry);
+
+    if (end_timestamp)
+    {
+        /* Check if we have reached the end of given journal range */
+        if (end_timestamp >= entry_timestamp)
+            return FALSE;
+    }
+
+    return TRUE;
+}
+
+static gboolean
+gl_query_check_journal_end (GlQuery *query, GlJournalEntry *entry)
+{
+    return gl_journal_entry_check_journal_end (entry, query->end_timestamp);
+}
+
+void
+gl_query_set_journal_timestamp_range (GlQuery *query,
+                                      guint64 start_timestamp,
+                                      guint64 end_timestamp)
+{
+    query->start_timestamp = start_timestamp;
+    query->end_timestamp = end_timestamp;
+}
+
 static gboolean
 is_string_case_sensitive (const gchar *user_text)
 {
@@ -742,7 +802,7 @@ calculate_match (GlJournalEntry *entry,
 
 
         /* check for matches */
-        token_match = gl_query_item_new (field_name, field_value, SEARCH_TYPE_SUBSTRING);
+        token_match = gl_query_item_new (field_name, field_value, GL_QUERY_SEARCH_TYPE_SUBSTRING);
 
         field_name_case = is_string_case_sensitive (field_name);
         field_value_case = is_string_case_sensitive (field_value);
@@ -843,28 +903,36 @@ search_in_entry (GlJournalEntry *entry,
 
     search_matches = gl_query_get_substring_matches (query);
 
-    /* Get search text from a search match */
-    search_match = g_ptr_array_index (search_matches, 0);
-
-    /* check for null and empty strings */
-    if (!search_match->field_value || !*(search_match->field_value))
+    /* Check if there is atleast one substring queryitem */
+    if (search_matches->len)
     {
-        matches = TRUE;
+        /* Get search text from a search match */
+        search_match = g_ptr_array_index (search_matches, 0);
+
+        /* check for null and empty strings */
+        if (!search_match->field_value || !*(search_match->field_value))
+        {
+            matches = TRUE;
+        }
+        else
+        {
+            gchar *search_text;
+
+            search_text = search_match->field_value;
+
+            /* Tokenize the entered text */
+            token_array = tokenize_search_string (search_text);
+
+            /* calculate match depending on the number of tokens */
+            matches = calculate_match (entry, token_array, search_matches);
+
+            /* Free variables */
+            g_ptr_array_free (token_array, TRUE);
+        }
     }
     else
     {
-        gchar *search_text;
-
-        search_text = search_match->field_value;
-
-        /* Tokenize the entered text */
-        token_array = tokenize_search_string (search_text);
-
-        /* calculate match depending on the number of tokens */
-        matches = calculate_match (entry, token_array, search_matches);
-
-        /* Free variables */
-        g_ptr_array_free (token_array, TRUE);
+        matches = TRUE;
     }
 
     g_ptr_array_free (search_matches, TRUE);
